#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: GPL-3.0-only

PKG_NAME = "fosslight_prechecker"
DEFAULT_EXCLUDE_EXTENSION = ["jar", "png", "exe", "so", "a", "dll", "jpeg", "jpg", "ttf", "lib", "ttc", "pfb",
                             "pfm", "otf", "afm", "dfont", "json"]
OSS_PKG_INFO_FILES = [r"oss-pkg-info[\s\S]*.ya?ml", r"oss-package[\s\S]*.info", r"sbom-info[\s\S]*.ya?ml",
                      "requirement.txt", "requirements.txt", "package.json", "pom.xml", "build.gradle",
                      "podfile.lock", "cartfile.resolved", "pubspec.yaml", "package.resolved", "go.mod",
                      "packages.config", "project.assets.json"]
HTML_RESULT_EXPAND_LIMIT = 10
HTML_RESULT_PRINT_LIMIT = 100

HTML_FORMAT_PREFIX = """
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/1999/REC-html401-19991224/strict.dtd">
<html lang="ko">
    <head>
        <meta http-equiv="content-type" content="text/html; charset=UTF-8">
        <style>
            .marker{background-color: Yellow;}
            .underline{text-decoration-line: underline;}
            table{width: 100%;}
        </style>
        <title>FOSSLight Prechecker</title>
    </head>
    <body>
            <table cellspacing="0" cellpadding="0" border="0" style="font-family:'arial,sans-serif'">
                <tr>
                    <td>
                        <table cellspacing="0" cellpadding="0" border="0">
                            <tr>
                                <td colspan="3" height="25" style="background:#c00c3f">
                                    <h1 style="margin:0;padding-left:5px;font-size:14px;color:white">FOSSLight Prechecker Lint</h1>
                                </td>
                            </tr>
                            <tr>
                                <td width="1" style="background:#ddd"></td>
                                <td style="padding:40px;">
                                    <div style="padding-bottom:10px;font-size:16px;font-weight:bold;">"""

HTML_COMPLIANCE_SUFFIX = "</div>"
HTML_CELL_HEAD_ROW = """<h2 style="margin:20px 0 0;padding:10px;font-size:16px;">« Files without License or Copyright »</h2>"""
HTML_CELL_PREFIX = """
                                <table cellspacing="0" cellpadding="0" width="100%" border="1" style="font-size:12px;border-color:#ddd;">
                                    <tr>
                                        <th style="padding:5px;background:#f0f0f0;">File</th>
                                        <th style="padding:5px;background:#f0f0f0;">License</th>
                                        <th style="padding:5px;background:#f0f0f0;">Copyright</th>
                                    </tr>"""

HTML_FORMAT_SUFFIX = """
                                    </table>
                                    <br/>
                                </td>
                                <td width="1" style="background:#ddd"></td>
                            </tr>
                            <tr>
                                <td colspan="3" height="1" style="background:#ddd"></td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
    </body>
</html>
"""

HTML_EXPAND_PREFIX = """<details style="font-size:12px;">
                            <summary style="font-size:12px;color:blue;"  class='underline'>Click to expand...</summary>"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: GPL-3.0-only
import logging
import os
from fosslight_util.constant import LOGGER_NAME
from fosslight_util.parsing_yaml import parsing_yml
from fosslight_util.output_format import write_output_file

logger = logging.getLogger(LOGGER_NAME)

HEADER_CONTENT = ['ID', 'Source Name or Path', 'OSS Name',
                  'OSS Version', 'License',  'Download Location',
                  'Homepage', 'Copyright Text', 'Exclude',
                  'Comment']
MAX_SHEET_NAME_LEN = 31


def get_sheet_name(yaml_file, sheet_list):
    if len(yaml_file) > MAX_SHEET_NAME_LEN:
        yaml_file = yaml_file[0:MAX_SHEET_NAME_LEN]

    count = 1
    while yaml_file in sheet_list:
        end_idx = MAX_SHEET_NAME_LEN - 1 - len(str(count))
        yaml_file = f"{yaml_file[0:end_idx]}_{count}"
        count += 1
        if end_idx == 0:
            yaml_file = ""
            logger.error("Too many duplicated file names")
            break
    return yaml_file


def convert_yml_to_excel(oss_yaml_files, output_file, file_option_on, base_path):
    sheet_list = {}
    header = {}

    for yaml_file in oss_yaml_files:
        try:
            if os.path.isfile(yaml_file):
                items_to_print = []

                logger.info(f"Read data from : {yaml_file}")

                if file_option_on:
                    base_path = os.path.dirname(yaml_file)
                oss_items, _ = parsing_yml(yaml_file, base_path)
                for item in oss_items:
                    items_to_print.extend(item.get_print_array())
                if not base_path.endswith(f"{os.sep}"):
                    base_path += f"{os.sep}"
                yaml_file = yaml_file.replace(base_path, '')
                yaml_file = yaml_file.replace(os.sep, '_')
                yaml_file_sheet = get_sheet_name(yaml_file, sheet_list)

                if yaml_file_sheet:
                    sheet_list[yaml_file_sheet] = items_to_print
                    header[yaml_file_sheet] = HEADER_CONTENT
        except Exception as ex:
            logger.error(f"Read yaml file: {ex}")

    try:
        success, msg, result_file = write_output_file(output_file, '.xlsx', sheet_list, header)
        if success:
            if result_file:
                logger.warning(f"Output: {result_file}")
            else:
                logger.warning("Nothing is detected to convert so output file is not generated.")
        else:
            logger.error(f"Fail to write excel file : {msg}")
    except Exception as ex:
        logger.error(f"Error to write excel file : {ex}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: GPL-3.0-only
import os
import sys
import shutil
import logging
import locale
import platform
import re
import subprocess
from datetime import datetime
from binaryornot.check import is_binary
import fosslight_util.constant as constant
from fosslight_util.set_log import init_log
from fosslight_util.timer_thread import TimerThread
from reuse import report
from reuse.project import Project
from reuse.report import ProjectReport
from reuse.vcs import VCSStrategyGit
from fosslight_prechecker._result import write_result_file, create_result_file, result_for_summary, ResultItem
from fosslight_prechecker._constant import DEFAULT_EXCLUDE_EXTENSION, OSS_PKG_INFO_FILES, PKG_NAME

is_windows = platform.system() == 'Windows'
REUSE_CONFIG_FILE = ".reuse/dep5"
DEFAULT_EXCLUDE_EXTENSION_FILES = []  # Exclude files from reuse
_turn_on_exclude_config = True
_check_only_file_mode = False
error_items = []
_start_time = ""
_result_log = {}

logger = logging.getLogger(constant.LOGGER_NAME)


def exclude_untracked_files(path):
    global DEFAULT_EXCLUDE_EXTENSION_FILES
    try:
        cmd_result = subprocess.check_output(['git', 'ls-files', '-o'], universal_newlines=True)
        cmd_result = cmd_result.split('\n')
        cmd_result.remove('')
        if not path.endswith(f"{os.sep}"):
            path += f"{os.sep}"
        cmd_result = [file.replace(path, '', 1) for file in cmd_result]
        DEFAULT_EXCLUDE_EXTENSION_FILES.extend(cmd_result)
    except Exception as ex:
        logger.error(f"Error to get git untracked files : {ex}")


def exclude_gitignore_files(current_path, path):
    global DEFAULT_EXCLUDE_EXTENSION_FILES
    try:
        root_path = VCSStrategyGit.find_root(current_path)
        if os.path.isfile(os.path.join(root_path, '.gitignore')):
            cmd_result = subprocess.check_output(['git',
                                                  'ls-files',
                                                  '-i',
                                                  '--exclude-from=.gitignore'],
                                                 universal_newlines=True)
            cmd_result = cmd_result.split('\n')
            cmd_result.remove('')
            if not path.endswith(f"{os.sep}"):
                path += f"{os.sep}"
            cmd_result = [file.replace(path, '', 1) for file in cmd_result]
            DEFAULT_EXCLUDE_EXTENSION_FILES.extend(cmd_result)
        else:
            return

    except Exception as ex:
        logger.error(f"Error to get git ignored files : {ex}")


def exclude_git_related_files(path):
    try:
        # Change currnt path for git command
        current_path = os.getcwd()
        os.chdir(path)

        # Exclude untracked files
        exclude_untracked_files(path)
        # Exclude ignore files
        exclude_gitignore_files(current_path, path)

        # Restore path
        os.chdir(current_path)
    except Exception as ex:
        logger.error(f"Error to get git related files : {ex}")


def find_oss_pkg_info_and_exlcude_file(path, mode='lint'):
    global DEFAULT_EXCLUDE_EXTENSION_FILES
    oss_pkg_info = []
    git_present = shutil.which("git")

    if mode == 'lint' and _turn_on_exclude_config and git_present and VCSStrategyGit.in_repo(path):
        exclude_git_related_files(path)

    try:
        for root, dirs, files in os.walk(path):
            for dir in dirs:
                # For hidden folders
                if dir.startswith("."):
                    all_files_in_dir = [os.path.join(root, dir, file)
                                        for file in os.listdir(os.path.join(root, dir))
                                        if os.path.isfile(os.path.join(root, dir, file))]
                    all_files_rel_path = [os.path.relpath(file, path) for file in all_files_in_dir]
                    DEFAULT_EXCLUDE_EXTENSION_FILES.extend(all_files_rel_path)

            for file in files:
                file_lower_case = file.lower()
                file_abs_path = os.path.join(root, file)
                file_rel_path = os.path.relpath(file_abs_path, path)

                if any(re.search(re_oss_pkg_pattern, file_lower_case) for re_oss_pkg_pattern in OSS_PKG_INFO_FILES) \
                   or file_lower_case.startswith("module_license_"):
                    oss_pkg_info.append(file_rel_path)
                # Exclude hidden files
                elif _turn_on_exclude_config and file.startswith('.'):
                    DEFAULT_EXCLUDE_EXTENSION_FILES.append(file_rel_path)
                elif is_binary(file_abs_path):
                    DEFAULT_EXCLUDE_EXTENSION_FILES.append(file_rel_path)
                else:
                    extension = file_lower_case.split(".")[-1]
                    if extension in DEFAULT_EXCLUDE_EXTENSION:
                        DEFAULT_EXCLUDE_EXTENSION_FILES.append(file_rel_path)
    except Exception as ex:
        dump_error_msg(f"Error_FIND_OSS_PKG : {ex}")

    return oss_pkg_info


def create_reuse_dep5_file(path):
    global DEFAULT_EXCLUDE_EXTENSION_FILES
    # Create .reuse/dep5 for excluding directories from reuse.
    _DEFAULT_CONFIG_PREFIX = "Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/\nUpstream-Name: \
                        reuse\nUpstream-Contact: Carmen Bianca Bakker <carmenbianca@fsfe.org>\nSource: https://github.com/fsfe/reuse-tool\n"
    _DEFAULT_EXCLUDE_FOLDERS = ["venv*/*", "node_modules*/*", ".*/*"]

    reuse_config_file = os.path.join(path, REUSE_CONFIG_FILE)
    file_to_remove = reuse_config_file
    dir_to_remove = os.path.dirname(reuse_config_file)
    need_rollback = False
    str_contents = ""

    try:
        if not os.path.exists(dir_to_remove):
            os.makedirs(dir_to_remove, exist_ok=True)
        else:
            dir_to_remove = ""
        if os.path.exists(reuse_config_file):
            file_to_remove = f"{reuse_config_file}_{_start_time}.bk"
            shutil.copy2(reuse_config_file, file_to_remove)
            need_rollback = True

        DEFAULT_EXCLUDE_EXTENSION_FILES.extend(_DEFAULT_EXCLUDE_FOLDERS)
        exclude_file_list = list(set(DEFAULT_EXCLUDE_EXTENSION_FILES))

        if is_windows:
            exclude_file_list = [file.replace(os.sep, '/') for file in exclude_file_list]

        for file_to_exclude in exclude_file_list:
            str_contents += f"\nFiles: {file_to_exclude} \nCopyright: -\nLicense: -\n"

        with open(reuse_config_file, "a", encoding="utf8") as f:
            if not need_rollback:
                f.write(_DEFAULT_CONFIG_PREFIX)
            f.write(str_contents)
    except Exception as ex:
        dump_error_msg(f"Error_Create_Dep5 : {ex}")

    return need_rollback, file_to_remove, dir_to_remove


def remove_reuse_dep5_file(rollback, file_to_remove, temp_dir_name):
    try:
        if rollback:
            _origin_file = os.path.join(os.path.dirname(file_to_remove), os.path.basename(REUSE_CONFIG_FILE))
            shutil.copy2(file_to_remove, _origin_file)

        os.remove(file_to_remove)

        if temp_dir_name != "":
            os.rmdir(temp_dir_name)

    except Exception as ex:
        dump_error_msg(f"Error_Remove_Dep5 : {ex}")


def precheck_for_files(path, files):
    global DEFAULT_EXCLUDE_EXTENSION_FILES

    missing_license_list = []
    missing_copyright_list = []

    try:
        prj = Project(path)

        for file in files:
            try:
                file_abs_path = os.path.join(path, file)
                if not os.path.isfile(file_abs_path) or is_binary(file_abs_path):
                    if is_windows:
                        file = file.replace(os.sep, '/')
                    DEFAULT_EXCLUDE_EXTENSION_FILES.append(file)
                else:
                    extension = file.split(".")[-1]
                    if extension in DEFAULT_EXCLUDE_EXTENSION:
                        if is_windows:
                            file = file.replace(os.sep, '/')
                        DEFAULT_EXCLUDE_EXTENSION_FILES.append(file)
                    else:
                        logger.info(f"# {file}")
                        rep = report.FileReport.generate(prj, file_abs_path)

                        logger.info(f"* License: {', '.join(rep.spdxfile.licenses_in_file)}")
                        logger.info(f"* Copyright: {rep.spdxfile.copyright}\n")

                        if rep.spdxfile.licenses_in_file is None or len(rep.spdxfile.licenses_in_file) == 0:
                            missing_license_list.append(file)
                        if rep.spdxfile.copyright is None or len(rep.spdxfile.copyright) == 0:
                            missing_copyright_list.append(file)

            except Exception as ex:
                dump_error_msg(f"Error - precheck_for_files to read : {ex}", True)

    except Exception as ex:
        dump_error_msg(f"Error - precheck_for_files: {ex}", True)

    return missing_license_list, missing_copyright_list, prj


def precheck_for_project(path_to_find, mode='lint'):
    missing_license = []
    missing_copyright = []

    oss_pkg_info_files = find_oss_pkg_info_and_exlcude_file(path_to_find, mode)
    if _turn_on_exclude_config:
        need_rollback, temp_file_name, temp_dir_name = create_reuse_dep5_file(path_to_find)

    try:
        project = Project(path_to_find)
        report = ProjectReport.generate(project)

        # File list that missing license text
        missing_license = [str(sub) for sub in set(report.files_without_licenses)]
        if not path_to_find.endswith(f"{os.sep}"):
            path_to_find += f"{os.sep}"
        missing_license = [sub.replace(path_to_find, '') for sub in missing_license]

        # File list that missing copyright text
        missing_copyright = [str(sub) for sub in set(report.files_without_copyright)]
        if not path_to_find.endswith(f"{os.sep}"):
            path_to_find += f"{os.sep}"
        missing_copyright = [sub.replace(path_to_find, '') for sub in missing_copyright]
    except Exception as ex:
        dump_error_msg(f"Error prechecker lint: {ex}", True)

    if _turn_on_exclude_config:
        remove_reuse_dep5_file(need_rollback, temp_file_name, temp_dir_name)
    return missing_license, missing_copyright, oss_pkg_info_files, project, report


def dump_error_msg(error_msg: str, exit=False):
    global error_items
    error_items.append(error_msg)
    if exit:
        logger.error(error_msg)
        sys.exit(1)


def init(path_to_find, output_path, file_list, need_log_file=True):
    global logger, _result_log
    logger, _result_log = init_log(os.path.join(output_path, f"fosslight_log_{_start_time}.txt"),
                                   need_log_file, logging.INFO, logging.DEBUG, PKG_NAME, path_to_find)
    if file_list:
        _result_log["File list to check"] = file_list


def get_path_to_find(target_path, _check_only_file_mode):
    is_file = False
    is_folder = False
    file_to_check_list = []
    path_list = []
    path_to_find = ""

    path_list = target_path.split(',')
    # Check if all elements are only files or folder
    for path in path_list:
        if os.path.isdir(path) and not is_folder:
            is_folder = True
            if path == "":
                path_to_find = os.getcwd()
            else:
                path_to_find = os.path.relpath(path)
        elif os.path.isfile(path):
            is_file = True
            path_to_find = os.getcwd()

    if is_file and is_folder:
        logger.error("Input only a folder or files with -p option")
    elif not is_folder and is_file:
        file_to_check_list = path_list
        _check_only_file_mode = True

    return path_to_find, file_to_check_list, _check_only_file_mode


def run_lint(target_path, disable, output_file_name, format='', need_log_file=True):
    global _turn_on_exclude_config, _check_only_file_mode, _start_time

    file_to_check_list = []
    _exit_code = 0
    path_to_find = ""
    report = ProjectReport()
    result_item = ResultItem()
    success = False
    _start_time = datetime.now().strftime('%y%m%d_%H%M')

    try:
        locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
    except Exception as ex:
        dump_error_msg(f"Error - locale : {ex}")

    path_to_find, file_to_check_list, _check_only_file_mode = get_path_to_find(target_path, _check_only_file_mode)

    result_file, output_path, output_extension = create_result_file(output_file_name, format, _start_time)
    init(path_to_find, output_path, file_to_check_list, need_log_file)

    if os.path.isdir(path_to_find):
        oss_pkg_info = []
        _turn_on_exclude_config = not disable

        if need_log_file:
            # Use ProgressBar
            timer = TimerThread()
            timer.setDaemon(True)
            timer.start()

        if _check_only_file_mode:
            license_missing_files, copyright_missing_files, project = precheck_for_files(path_to_find, file_to_check_list)
        else:
            license_missing_files, copyright_missing_files, oss_pkg_info, project, report = precheck_for_project(path_to_find, 'lint')

        if need_log_file:
            timer.stop = True

        result_item = result_for_summary(path_to_find,
                                         oss_pkg_info,
                                         license_missing_files,
                                         copyright_missing_files,
                                         report,
                                         _result_log,
                                         _check_only_file_mode,
                                         file_to_check_list,
                                         error_items,
                                         DEFAULT_EXCLUDE_EXTENSION_FILES)

        success, exit_code = write_result_file(result_file, output_extension, _exit_code,
                                               result_item, _result_log, project, path_to_find)
        if success:
            logger.warning(f"Created file name: {result_file}\n")
        else:
            logger.warning("Can't make result file\n")
        sys.exit(exit_code)
    else:
        logger.error(f"Check the path to find : {target_path}")
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: GPL-3.0-only
import os
import io
import sys
import platform
import yaml
import fnmatch
import xml.etree.ElementTree as ET
import logging
import fosslight_util.constant as constant
import re
from pathlib import Path
from reuse.project import Project
from fosslight_prechecker._result_html import result_for_html
from fosslight_util.parsing_yaml import find_sbom_yaml_files, parsing_yml
from fosslight_util.output_format import check_output_format


CUSTOMIZED_FORMAT_FOR_PRECHECKER = {'html': '.html', 'xml': '.xml', 'yaml': '.yaml'}
RULE_LINK = "https://oss.lge.com/guide/process/osc_process/1-identification/copyright_license_rule.html"
MSG_REFERENCE = "Ref. Copyright and License Writing Rules in Source Code. : " + RULE_LINK
MSG_FOLLOW_LIC_TXT = "Follow the Copyright and License Writing Rules in Source Code. : " + RULE_LINK
EX_IOERR = 74
logger = logging.getLogger(constant.LOGGER_NAME)
is_windows = platform.system() == 'Windows'


class ResultItem:
    def __init__(self):
        self._compliant_result = ""
        self._summary = ""
        self._oss_pkg_files = []
        self._detected_licenses = []
        self._count_total_files = ""
        self._count_without_both = ""
        self._count_without_lic = ""
        self._count_without_cop = ""
        self._files_without_both = []
        self._files_without_lic = []
        self._files_without_cop = []
        self._os_info = ""
        self._path_to_analyze = ""
        self._python_ver = ""
        self._fl_prechecker_ver = ""
        self._log_msg = ""
        self._check_only_file_mode = False
        self.execution_error = []

    @property
    def compliant_result(self):
        return self._compliant_result

    @compliant_result.setter
    def compliant_result(self, value):
        if value:
            self._compliant_result = "OK"
        else:
            self._compliant_result = "Not-OK"

    @property
    def oss_pkg_files(self):
        return self._oss_pkg_files

    def get_print_yaml(self):
        result_item = {}
        result_item["Compliant"] = self._compliant_result
        result_summary_item = {}
        result_summary_item["Open Source Package File"] = is_list_empty(self._oss_pkg_files)
        result_summary_item["Detected Licenses"] = is_list_empty(self._detected_licenses)
        result_summary_item["Files without license / total"] = f"{self._count_without_lic} / {self._count_total_files}"
        result_summary_item["Files without copyright / total"] = f"{self._count_without_cop} / {self._count_total_files}"

        result_item["Summary"] = result_summary_item
        result_item["Files without license and copyright"] = is_list_empty(self._files_without_both)
        result_item["Files without license"] = is_list_empty(self._files_without_lic)
        result_item["Files without copyright"] = is_list_empty(self._files_without_cop)

        result_tool_item = {}
        result_tool_item["OS"] = self._os_info
        result_tool_item["Analyze path"] = self._path_to_analyze
        result_tool_item["Python version"] = self._python_ver
        result_tool_item["fosslight_prechecker version"] = self._fl_prechecker_ver
        result_item["Tool Info"] = result_tool_item
        if self.execution_error:
            result_item["Execution Error"] = self.execution_error
        root_item = {"Checking copyright/license writing rules": result_item}
        return root_item


def is_list_empty(list: list):
    if list:
        return list
    else:
        return "N/A"


def result_for_xml(result_item: ResultItem):
    str_xml_result = ""
    _root_xml_item = ET.Element('results')

    if result_item.compliant_result != "OK":
        if not result_item._check_only_file_mode:
            _SUMMARY_PREFIX = "\n# SUMMARY\n"
            str_xml_result = f"* Open Source Package File: {', '.join(result_item._oss_pkg_files)}\n \
                               * Detected Licenses : {', '.join(result_item._detected_licenses)}\n \
                               * Files without copyright : {result_item._count_without_cop} / {result_item._count_total_files}\n \
                               * Files without license : {result_item._count_without_lic} / {result_item._count_total_files}\n"

            _SUMMARY_SUFFIX = '\n\n' + MSG_REFERENCE
            str_summary = f"{_SUMMARY_PREFIX}{str_xml_result}{_SUMMARY_SUFFIX}"
            items = ET.Element('error')
            items.set('id', 'rule_key_osc_checker_01')
            items.set('line', '0')
            items.set('msg', str_summary)
            _root_xml_item.append(items)
        else:
            for file_name in (set(result_item._files_without_lic) | set(result_item._files_without_cop)):
                items = ET.Element('error')
                items.set('file', file_name)
                items.set('id', 'rule_key_osc_checker_02')
                items.set('line', '0')
                items.set('msg', MSG_FOLLOW_LIC_TXT)
                _root_xml_item.append(items)

            for file_name in result_item._files_without_cop:
                items = ET.Element('error')
                items.set('file', file_name)
                items.set('id', 'rule_key_osc_checker_02')
                items.set('line', '0')
                items.set('msg', MSG_FOLLOW_LIC_TXT)
                _root_xml_item.append(items)

    if result_item.execution_error:
        error_xml = ET.Element('execution_errors')
        for error in result_item.execution_error:
            error_xml_sub = ET.SubElement(error_xml, 'execution_error')
            error_xml_sub.text = error
        _root_xml_item.append(error_xml)
    return _root_xml_item


def write_result_xml(result_file: str, exit_code: int, result_item: ResultItem, _result_log: str):
    success = False
    # Create a new XML file with the results
    try:
        output_dir = os.path.dirname(result_file)
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        _root_xml_item = result_for_xml(result_item)
        ET.ElementTree(_root_xml_item).write(result_file, encoding="UTF-8", xml_declaration=True)
        success = True
    except Exception as ex:
        logger.error(f"Error_to_write_xml: {ex}")
        exit_code = EX_IOERR
    return success, exit_code


def write_result_html(result_file: str, exit_code: int, result_item: ResultItem, project: Project, path_to_find):
    success = False
    try:
        output_dir = os.path.dirname(result_file)
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        html_result = result_for_html(result_item, project, path_to_find)
        with open(result_file, 'w') as f:
            f.write(html_result)
        success = True
    except Exception as ex:
        logger.error(f"Error_to_write_html: {ex}")
        exit_code = EX_IOERR
    return success, exit_code


def write_result_yaml(result_file: str, exit_code: int, result_item: ResultItem):
    success = False
    try:
        output_dir = os.path.dirname(result_file)
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        yaml_result = result_item.get_print_yaml()
        # Make yaml file
        with io.open(result_file, 'w', encoding='utf8') as outfile:
            yaml.dump(yaml_result, outfile, default_flow_style=False, sort_keys=False, allow_unicode=True)
        success = True
    except Exception as ex:
        logger.error(f"Error_to_write_yaml: {ex}")
        exit_code = EX_IOERR
    return success, exit_code


def create_result_file(output_file_name, format='', _start_time=""):
    success, msg, output_path, output_file, output_extension = check_output_format(output_file_name, format, CUSTOMIZED_FORMAT_FOR_PRECHECKER)
    if success:
        result_file = ""
        if output_path == "":
            output_path = os.getcwd()
        else:
            output_path = os.path.abspath(output_path)

        if output_file != "":
            result_file = f"{output_file}{output_extension}"
        else:
            if output_extension == '.yaml' or output_extension == "":
                result_file = f"fosslight_lint_{_start_time}.yaml"
            elif output_extension == '.html':
                result_file = f"fosslight_lint_{_start_time}.html"
            elif output_extension == '.xml':
                result_file = f"fosslight_lint_{_start_time}.xml"
            else:
                logger.error("Not supported file extension")

        result_file = os.path.join(output_path, result_file)
    else:
        logger.error(f"Format error - {msg}")
        sys.exit(1)

    return result_file, output_path, output_extension


def get_path_in_yaml(oss_item):
    path_in_yaml = [os.path.join(oss_item.relative_path, file) for file in oss_item.source_name_or_path]
    if is_windows:
        path_in_yaml = [path.replace(os.sep, '/') for path in path_in_yaml]
    return path_in_yaml


def extract_files_in_path(remove_file_list, base_file_list, return_found=False):
    extract_files = []
    remained_file_to_remove = []
    remained_base_files = []

    if base_file_list:
        intersection_files = list(set(base_file_list) & set(remove_file_list))
        extract_files.extend(intersection_files)
        remained_base_files = list(set(base_file_list) - set(intersection_files))
        remained_file_to_remove = list(set(remove_file_list) - set(intersection_files))

    for remove_pattern in remained_file_to_remove:
        for file in remained_base_files[:]:
            if fnmatch.fnmatch(file, remove_pattern) or re.search(remove_pattern, file):
                extract_files.append(file)
                remained_base_files.remove(file)
    return extract_files if return_found else remained_base_files


def exclude_file_in_yaml(path_to_find, yaml_files, license_missing_files, copyright_missing_files):
    excluded_path = []
    lic_present_path = []
    cop_present_path = []
    for file in yaml_files:
        oss_items, _ = parsing_yml(file, path_to_find)
        for oss_item in oss_items:
            if oss_item.exclude:
                excluded_path.extend(get_path_in_yaml(oss_item))
            else:
                if oss_item.license:
                    lic_present_path.extend(get_path_in_yaml(oss_item))
                if oss_item.copyright:
                    cop_present_path.extend(get_path_in_yaml(oss_item))

    total_missing_files = list(license_missing_files.union(copyright_missing_files))
    files_with_exclude_removed = extract_files_in_path(excluded_path, total_missing_files, True)
    license_missing_files = extract_files_in_path(lic_present_path, list(license_missing_files - set(files_with_exclude_removed)))
    copyright_missing_files = extract_files_in_path(cop_present_path, list(copyright_missing_files - set(files_with_exclude_removed)))

    return license_missing_files, copyright_missing_files


def result_for_summary(path_to_find, oss_pkg_info_files, license_missing_files, copyright_missing_files,
                       prj_report, _result_log, _check_only_file_mode, file_to_check_list, error_items, exclude_files):
    prechecker_compliant = False
    detected_lic = []
    missing_both_files = []
    file_total_num = ""
    total_files_exclude = ()

    if _check_only_file_mode:
        file_total_num = len(file_to_check_list)
    else:
        if not path_to_find.endswith('/'):
            path_to_find += '/'
        total_files = [str(file_report.path).replace(path_to_find, '', 1) for file_report in prj_report.file_reports]
        total_files_exclude = set(total_files) - set(exclude_files)
        file_total_num = len(total_files_exclude)

        # Get detected License
        for i, lic in enumerate(sorted(prj_report.used_licenses)):
            detected_lic.append(lic)

    if oss_pkg_info_files:
        oss_yaml_files = find_sbom_yaml_files(path_to_find)
        # Exclude files in yaml
        license_missing_files, copyright_missing_files = exclude_file_in_yaml(path_to_find, oss_yaml_files,
                                                                              set(license_missing_files) - set(oss_pkg_info_files),
                                                                              set(copyright_missing_files) - set(oss_pkg_info_files))
        # Subtract excluded files(untracked or ignored file)
        oss_pkg_info_files = list(set(oss_pkg_info_files) - set(exclude_files))

    if len(license_missing_files) == 0 and len(copyright_missing_files) == 0:
        prechecker_compliant = True

    # Remove duplicated file
    missing_both_files = list(set(license_missing_files) & set(copyright_missing_files))
    license_missing_files = list(set(license_missing_files) - set(missing_both_files))
    copyright_missing_files = list(set(copyright_missing_files) - set(missing_both_files))

    # Save result items
    result_item = ResultItem()
    result_item.compliant_result = prechecker_compliant
    result_item._oss_pkg_files = oss_pkg_info_files
    result_item._detected_licenses = detected_lic
    result_item._count_total_files = file_total_num
    result_item._count_without_both = str(len(missing_both_files))
    result_item._count_without_lic = str(len(license_missing_files) + len(missing_both_files))
    result_item._count_without_cop = str(len(copyright_missing_files) + len(missing_both_files))
    result_item._files_without_both = sorted(missing_both_files)
    result_item._files_without_lic = sorted(license_missing_files)
    result_item._files_without_cop = sorted(copyright_missing_files)
    result_item._fl_prechecker_ver = _result_log["Tool Info"]
    result_item._path_to_analyze = _result_log["Path to analyze"]
    result_item._os_info = _result_log["OS"]
    result_item._python_ver = _result_log["Python version"]
    result_item._check_only_file_mode = _check_only_file_mode
    result_item.execution_error = error_items
    return result_item


def write_result_file(result_file, output_extension, exit_code, result_item, _result_log, project, path_to_find):
    success = False
    if output_extension == ".yaml" or output_extension == "":
        success, exit_code = write_result_yaml(result_file, exit_code, result_item)
    elif output_extension == ".html":
        success, exit_code = write_result_html(result_file, exit_code, result_item, project, path_to_find)
    elif output_extension == ".xml":
        success, exit_code = write_result_xml(result_file, exit_code, result_item, _result_log)
    else:
        logger.info("Not supported file extension")

    if success:
        # Print yaml result
        yaml_result = result_item.get_print_yaml()
        str_yaml_result = yaml.safe_dump(yaml_result, allow_unicode=True, sort_keys=False)
        logger.info(str_yaml_result)
    return success, exit_code
